package cn.skcks.study.springcloud.kafka.tests;

import cn.skcks.study.springcloud.kafka.TestApplication;
import cn.skcks.study.springcloud.utils.JsonUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.admin.*;
import org.apache.kafka.common.config.ConfigResource;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.kafka.KafkaProperties;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import javax.annotation.PostConstruct;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;

@ActiveProfiles("test")
@SpringBootTest(classes = TestApplication.class, webEnvironment = SpringBootTest.WebEnvironment.NONE)
@Slf4j
public class KafkaConfigTest {
    @Autowired
    private KafkaProperties kafkaProperties;

    private final static String TOPIC = "test";

    private Map<String, Object> adminProperties;
    private Map<String, Object> producerProperties;
    private Map<String, Object> consumerProperties;
    private Map<String, Object> streamsProperties;

    @PostConstruct
    void init() {
        adminProperties = kafkaProperties.buildAdminProperties();
        producerProperties = kafkaProperties.buildProducerProperties();
        consumerProperties = kafkaProperties.buildConsumerProperties();
        streamsProperties = kafkaProperties.buildStreamsProperties();
    }

    @Test
    void properties() {
        log.info("AdminProperties {}", JsonUtils.toJson(adminProperties));
        log.info("ProducerProperties {}", JsonUtils.toJson(producerProperties));
        log.info("ConsumerProperties {}", JsonUtils.toJson(consumerProperties));
        log.info("StreamsProperties {}", JsonUtils.toJson(streamsProperties));
    }

    private AdminClient adminClient() {
        return AdminClient.create(kafkaProperties.buildAdminProperties());
    }

    @Test
    @SneakyThrows
    void topics() {
        try (AdminClient client = adminClient()) {
            ListTopicsOptions options = new ListTopicsOptions();
            // 包含内置主题
            options.listInternal(true);

            ListTopicsResult topicsResult = client.listTopics(options);
            topicsResult.listings().get().stream().sorted((a, b) -> {
                if (a.isInternal() && b.isInternal()) {
                    return 0;
                }
                return a.isInternal() ? -1 : 1;
            }).forEach(topic -> {
                log.info("topic isInternal=>{}, uuid=>{}, name=>{}", topic.isInternal(), topic.topicId(), topic.name());
            });
        }
    }

    @Test
    @SneakyThrows
    void testTopicInfo() {
        try (AdminClient client = adminClient()) {
            ConfigResource resource = new ConfigResource(ConfigResource.Type.TOPIC, TOPIC);
            log.info("原配置");
            getTopicConfig(client, resource);

            log.info("修改配置");
            alterTopicConfig(client, resource);

            log.info("修改后配置");
            getTopicConfig(client, resource);
        }
    }

    @SneakyThrows
    void getTopicConfig(AdminClient client, ConfigResource resource) {
        DescribeConfigsResult describeConfigsResult = client.describeConfigs(Collections.singletonList(resource));
        describeConfigsResult.all().get().forEach((source, config) -> {
            log.info("{} =>", source.name());
            config.entries().forEach(entry -> {
                log.info("\t{} => {}", entry.name(), entry.value());
            });
        });
    }

    @SneakyThrows
    void alterTopicConfig(AdminClient client, ConfigResource resource) {
        Collection<AlterConfigOp> configs = Arrays.asList(
                new AlterConfigOp(new ConfigEntry("min.cleanable.dirty.ratio", "0.5"), AlterConfigOp.OpType.SET),
                new AlterConfigOp(new ConfigEntry("unclean.leader.election.enable", "false"), AlterConfigOp.OpType.SET),
                // 设置30秒过期
                new AlterConfigOp(new ConfigEntry("retention.ms", "30000"), AlterConfigOp.OpType.SET),
                new AlterConfigOp(new ConfigEntry("segment.ms", "30000"), AlterConfigOp.OpType.SET)
        );

        AlterConfigsResult result = client.incrementalAlterConfigs(Collections.singletonMap(resource, configs));
        result.all().get();
    }
}
